Package: measles
Type: Package
Title: Measles Epidemiological Models
Version: 0.2.0
Depends: R (>= 4.1.0), epiworldR (>= 0.14.0.0)
Authors@R: c(
  person(given="George", family="Vega Yon", role=c("aut", "cre"),
  email="g.vegayon@gmail.com", comment = c(ORCID = "0000-0002-3171-0844")),
  person(
    "Damon", "Toth", email = "damon.toth@hsc.utah.edu", role = c("ctb"),
    comment = c(ORCID = "0000-0001-7393-4814")
    ),
  person(
    "Jake", "Wagoner", email = "jakew@sci.utah.edu", role = c("ctb"),
    comment = c(ORCID = "0009-0000-5053-2281")
  ),
  person(
    "Olivia", "Banks", email = "olivia.banks@utah.edu", role = c("ctb"),
    comment = c(ORCID = "0009-0008-7611-6030")
  ),
  person("Centers for Disease Control and Prevention", role="fnd", comment = "Award number 1U01CK000585; 75D30121F00003"
  ))
Description: A specialized collection of measles epidemiological models built on the 'epiworldR' framework. This package is a spinoff from 'epiworldR' focusing specifically on measles transmission dynamics. It includes models for school settings with quarantine and isolation policies, mixing models with population groups, and risk-based quarantine strategies. The models use Agent-Based Models (ABM) with a fast 'C++' backend from the 'epiworld' library. Ideal for studying measles outbreaks, vaccination strategies, and intervention policies.
URL: https://github.com/UofUEpiBio/measles
BugReports: https://github.com/UofUEpiBio/measles/issues
License: MIT + file LICENSE
RoxygenNote: 7.3.3
Encoding: UTF-8
LinkingTo: cpp11, epiworldR
SystemRequirements: C++17
Imports: utils
Suggests: tinytest, data.table, quarto
VignetteBuilder: quarto
LazyData: true
NeedsCompilation: yes
Packaged: 2026-03-31 21:50:26 UTC; runner
Author: George Vega Yon [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3171-0844>),
  Damon Toth [ctb] (ORCID: <https://orcid.org/0000-0001-7393-4814>),
  Jake Wagoner [ctb] (ORCID: <https://orcid.org/0009-0000-5053-2281>),
  Olivia Banks [ctb] (ORCID: <https://orcid.org/0009-0008-7611-6030>),
  Centers for Disease Control and Prevention [fnd] (Award number
    1U01CK000585; 75D30121F00003)
Maintainer: George Vega Yon <g.vegayon@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-31 23:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 21:52:37 UTC; unix
Archs: measles.so.dSYM
