# Generated by roxygen2: do not edit by hand

S3method(get_contact_matrix,default)
S3method(get_contact_matrix,epiworld_measlesmixing)
S3method(get_contact_matrix,epiworld_measlesmixingriskquarantine)
S3method(set_contact_matrix,default)
S3method(set_contact_matrix,epiworld_measlesmixing)
S3method(set_contact_matrix,epiworld_measlesmixingriskquarantine)
export(ModelMeaslesMixing)
export(ModelMeaslesMixingRiskQuarantine)
export(ModelMeaslesQuarantine)
export(ModelMeaslesSchool)
export(get_contact_matrix)
export(set_contact_matrix)
import(epiworldR)
importFrom(graphics,boxplot)
importFrom(graphics,plot)
importFrom(utils,packageVersion)
useDynLib(measles, .registration = TRUE)
